/*
   ATtiny85 sketch for helmet light.
   Program includes:
   Main lighting modes
   Other lighting modes
   Sleep mode
   Low battery monitor (goes to low power mode or sleep when battery is low)

   Holding the button
   In sleep mode for >0.3s turns on the helmet light
   While on for <=0.3s increments the mode.
   While on between >0.3 and <=0.9s puts the helmet light to sleep
   While on >0.9s increments the mode types (daytime, night time, and other)
*/

#include <avr/sleep.h>

// Utility macros
#define adc_disable() (ADCSRA &= ~(1<<ADEN)) // disable ADC
#define adc_enable()  (ADCSRA |=  (1<<ADEN)) // re-enable ADC

//Battery parameters
int batteryReading = 0;
int lowBatteryADC = 475; // Cutoff battery voltage is 10V. Cutoff feedback voltage is 2.33V. Calculated for the resistor values R9=330k, R10=100k.
int deadBatteryADC = 428; // Battery voltage is 9V. Feedback voltage is 2.09V

const int powerButtonPin = 2;
const int batteryMonitorPin = A2;

const int headlightPwmPin = 1;
const int rearPwmPin = 0;

/*const int headlightPwmPin = 0;
  const int rearPwmPin = 1;
*/
const int driverEnablePin = 3;

boolean powerButtonState;

int delayCounts;
int debounceDelay = 300;
// Recalls last setting.
int mode = 0;
int dayModesTop = 2;
int nightModesTop = 5;
int otherModesTop = 11;

int modeType = 0;
int modeTypeTop = 2;
boolean otherModesEnabled = false;

// 0 for headlight, 1 for tail light
int offTime0;
int offTime1;
int onTime0;
int onTime1;

int headLightOnTimeHigh;
int tailLightOnTimeHigh;

unsigned long elapsedMillis0 = 0;
unsigned long elapsedMillis1 = 0;
unsigned long prevMillis0 = 0;
unsigned long prevMillis1 = 0;

// Pattern parameters
int stepTime = 25; // Actual time in milliseconds is stepTime/68. Fade frequency is approximately 1 Hz.
int stepLevel;
int stepTop = 19;

// Four fade top pulse width levels
//int fadeLevelsEighth[20] = {0, 3, 6, 9, 12, 16, 19, 22, 25, 28, 31, 28, 25, 22, 19, 16, 12, 9, 6, 3};
int fadeLevelsQuarter[20] = {0, 6, 13, 19, 25, 32, 38, 44, 50, 57, 63, 57, 50, 44, 38, 32, 25, 19, 13, 6
                            };
/*  int fadeLevelsHalf[20] = {0, 13, 25, 38, 51, 64, 76, 89, 102, 114, 127, 114, 102, 89, 76, 64, 51, 38, 25, 13
                         };
  int fadeLevelsFull[20] = {0, 26,  51, 77, 102, 128, 153, 179, 204, 230, 255, 230, 204, 179, 153, 128, 102, 77, 51, 26};*/


void setup() {
  // put your setup code here, to run once:
  pinMode(headlightPwmPin, OUTPUT);
  pinMode(rearPwmPin, OUTPUT);
  pinMode(driverEnablePin, OUTPUT);
  pinMode(powerButtonPin, INPUT_PULLUP);
  pinMode(batteryMonitorPin, INPUT);

  sleepNow();
}

void loop() {
  readButtons();

  switch (mode) {

    // Day modes start here

    case 0: // Headlight flashes at quarter power
      headLightOnTimeHigh = 63;
      tailLightOnTimeHigh = 255;
      asyncDayPulse();
      break;
    case 1: // Headlight flashes at full power
      headLightOnTimeHigh = 255;
      tailLightOnTimeHigh = 255;
      asyncDayPulse();
      break;
    case 2:
      fourDayPulsesAsync();
      break;

    // Night modes start here
    case 3:
      headLightOnTimeHigh = 31; // ~12%
      invertedTripleFlashHeadlight();
      fadeRearLight();
      break;
    case 4:
      headLightOnTimeHigh = 127; // ~50%
      invertedTripleFlashHeadlight();
      fadeRearLight();
      break;
    case 5:
      headLightOnTimeHigh = 255; // 100%
      invertedTripleFlashHeadlight();
      fadeRearLight();
      break;

    // Other modes start here

    case 6:
      fiftyFifty18Hz();
      analogWrite(rearPwmPin, 255 - 0); // Disables the rear light
      break;
    case 7: // Solid rear only
      analogWrite(headlightPwmPin, 255 - 0); // Disables the headlight
      analogWrite(rearPwmPin, 255 - 31);
      break;
    case 8: // Front solid low
      analogWrite(headlightPwmPin, 255 - 31); // ~12%
      analogWrite(rearPwmPin, 255 - 0);
      break;
    case 9: // Front solid med
      analogWrite(headlightPwmPin, 255 - 127); // ~50%
      analogWrite(rearPwmPin, 255 - 0);
      break;
    case 10: // Front solid high
      analogWrite(headlightPwmPin, 255 - 255); // 100%
      analogWrite(rearPwmPin, 255 - 0);
      break;
    case 11: // Power saver mode. Allows you to have some visibility before the battery runs out
      powerSaverMode();
      break;
  }
  monitorBattery();
}

void monitorBattery() {
  // Measures the battery voltage
  batteryReading = analogRead(batteryMonitorPin);

  if (batteryReading < lowBatteryADC ) { // If battery is low, go to low power mode
    mode = 11;
  }
  if (batteryReading < deadBatteryADC) { // If batteyr is dead, go to sleep
    sleepNow();
  }
}

